#    Copyright (C) 2017 Cristiano Maruti (twitter: @cmaruti)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; Applies version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import re
import time
import urllib.request

__name__ = 'bingenum'

url_opener = urllib.request.FancyURLopener


class AppURLopener(url_opener):
    version = 'Mozilla/5.0 (compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm)'


def scrape_bing(dom):
    """
    Function for enumerating subdomains and hosts by scraping Bing.
    """
    results = []
    searches = [
        '10',
        '20',
        '30',
        '40',
        '50',
        '60',
        '70',
        '80',
        '90',
        '100',
        '110',
        '120',
        '130',
        '140',
        '150',
    ]
    urllib._urlopener = AppURLopener()

    for n in searches:
        url = 'https://www.bing.com/search?q=domain%3A' + dom + '&qs=n&first=' + n
        req = urllib.request.Request(
            url,
            data=None,
            headers={
                'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 Safari/537.36'
            },
        )

        sock = urllib.request.urlopen(req, timeout=10)
        data = sock.read().decode('utf-8')
        results.extend(re.findall(r'([a-zA-Z0-9\-.]+' + dom + ')/?', data))
        sock.close()
        time.sleep(5)

    return unique(results)


def unique(seq, idfun=repr):
    """
    Function to remove duplicates in an array. Returns array with duplicates
    removed.
    """
    seen = {}
    return [seen.setdefault(idfun(e), e) for e in seq if idfun(e) not in seen]



from unittest.mock import patch, MagicMock
from dnsrecon import cli


def test_check_wildcard():
    with patch('dnsrecon.lib.dnshelper.DnsHelper') as mock_dns_helper:
        mock_instance = mock_dns_helper.return_value
        mock_instance.check_wildcard.return_value = (True, ["192.0.2.1"])
        result = cli.check_wildcard(mock_instance, "zonetransfer.me")
        assert result == set()  # The function returns an empty set


def test_expand_range():
    input_range = "192.0.2.0"
    end_ip = "192.0.2.3"
    result = cli.expand_range(input_range, end_ip)
    assert len(result) == 4
    assert "192.0.2.0" in result
    assert "192.0.2.3" in result


def test_brute_domain():
    with patch('dnsrecon.lib.dnshelper.DnsHelper') as mock_dns_helper, \
            patch('builtins.input', return_value='y'):  # Mock user input
        mock_instance = mock_dns_helper.return_value
        mock_instance.get_a.return_value = ["192.0.2.1"]
        mock_instance.get_aaaa.return_value = ["2001:db8::1"]
        mock_instance.check_wildcard.return_value = (False, [])

        # Mock the generate_testname function to return a valid string
        with patch('dnsrecon.cli.generate_testname', return_value='testname.zonetransfer.me'):
            result = cli.brute_domain(mock_instance, "zonetransfer.me", ["subdomain"])

    assert isinstance(result, list)


def test_general_enum():
    with patch('dnsrecon.lib.dnshelper.DnsHelper') as mock_dns_helper:
        mock_instance = mock_dns_helper.return_value
        mock_instance.get_a.return_value = ["192.0.2.1"]
        mock_instance.get_aaaa.return_value = ["2001:db8::1"]
        mock_instance.get_mx.return_value = ["mail.zonetransfer.me"]
        mock_instance.get_txt.return_value = ["txt.zonetransfer.me"]
        mock_instance.get_ns.return_value = ["ns.zonetransfer.me"]
        mock_instance.get_soa.return_value = ["soa.zonetransfer.me"]
        mock_instance.get_srv.return_value = ["srv.zonetransfer.me"]
        mock_instance.get_spf.return_value = ["spf.zonetransfer.me"]
        mock_instance.get_nsec.return_value = ["nsec.zonetransfer.me"]
        mock_instance.get_nsec3.return_value = ["nsec3.zonetransfer.me"]
        mock_instance.get_nsec3param.return_value = ["nsec3param.zonetransfer.me"]
        mock_instance.get_ds.return_value = ["ds.zonetransfer.me"]
        mock_instance.get_dnskey.return_value = ["dnskey.zonetransfer.me"]
        mock_instance.get_rrsig.return_value = ["rrsig.zonetransfer.me"]
        result = cli.general_enum(mock_instance, "zonetransfer.me", True, True, True, True, True, True, True, 5.0)
        assert result is None  # The function doesn't return anything


def test_get_nsec_type():
    with patch('dnsrecon.lib.dnshelper.DnsHelper') as mock_dns_helper:
        mock_instance = mock_dns_helper.return_value
        mock_instance._res = MagicMock()
        mock_instance._res.nameservers = ["8.8.8.8"]
        mock_instance._res.timeout = 2.0

        mock_answer = MagicMock()
        mock_answer.authority = []

        with patch('dnsrecon.cli.get_a_answer', return_value=mock_answer):
            result = cli.get_nsec_type("zonetransfer.me", mock_instance)

        assert result is None



#!/usr/bin/env python3

#    Copyright (C) 2020  Carlos Perez
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; Applies version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import random
import socket

import dns.message
import dns.query
import dns.resolver
import dns.reversename
from dns.dnssec import algorithm_to_text
from dns.zone import *
from loguru import logger

DNS_PORT_NUMBER = 53
DNS_QUERY_TIMEOUT = 4.0


def strip_last_dot(addr_):
    """
    Util function that strips the last dot from an address (if any)
    """
    return addr_[:-1] if addr_.endswith('.') else addr_


class DnsHelper:
    def __init__(self, domain, ns_server=None, request_timeout=3.0, proto='tcp'):
        self._domain = domain
        self._proto = proto
        self._is_tcp = proto == 'tcp'

        configure = not ns_server
        self._res = dns.resolver.Resolver(configure=configure)

        if ns_server:
            if isinstance(ns_server, str):
                ns_server = [ns_server]
            self._res.nameservers = ns_server
            if len(ns_server) > 1:
                self._res.rotate = True

        # Set timing
        self._res.timeout = request_timeout
        self._res.lifetime = request_timeout

    def check_tcp_dns(self, address):
        """
        Function to check if a server is listening at port 53 TCP. This will aid
        in IDS/IPS detection since a AXFR will not be tried if port 53 is found to
        be closed.
        """
        try:
            sock = socket.socket()
            sock.settimeout(DNS_QUERY_TIMEOUT)
            sock.connect((address, DNS_PORT_NUMBER))
        except Exception:
            return False

        return True

    def get_answers(self, type_, addr_):
        """
        Function that wraps the resolve() function with all the specific
        exceptions it could raise and the socket.error exception
        https://dnspython.readthedocs.io/en/latest/resolver-class.html#dns.resolver.Resolver.resolve
        """
        try:
            return self._res.resolve(addr_, type_, tcp=self._is_tcp)
        except (
            OSError,
            dns.exception.Timeout,
            dns.resolver.NXDOMAIN,
            dns.resolver.YXDOMAIN,
            dns.resolver.NoAnswer,
            dns.resolver.NoNameservers,
            dns.name.EmptyLabel,
        ):
            return None

    def resolve(self, target, type_, ns=None):
        """
        Function for performing general resolution types returning the RDATA
        """
        configure = not ns
        res = dns.resolver.Resolver(configure=configure)
        if ns:
            res.nameservers = [ns]

        answers = res.query(target, type_, tcp=self._is_tcp)
        return answers

    def query(
        self,
        q,
        where,
        timeout=DNS_QUERY_TIMEOUT,
        port=53,
        af=None,
        source=None,
        source_port=0,
        one_rr_per_rrset=False,
    ):
        if isinstance(where, list):
            random.shuffle(where)
            target_server = where[0]
        else:
            target_server = where

        if self._is_tcp:
            return dns.query.tcp(
                q,
                target_server,
                timeout,
                port,
                af,
                source,
                source_port,
                one_rr_per_rrset,
            )
        else:
            return dns.query.udp(
                q,
                target_server,
                timeout,
                port,
                af,
                source,
                source_port,
                False,
                one_rr_per_rrset,
            )

    def get_a(self, host_trg):
        """
        Function for resolving the A Record for a given host. Returns an Array of
        the IP Address it resolves to. It will also return CNAME data.
        """
        answers = self.get_answers('A', host_trg)
        if not answers:
            return []

        result = []
        for answer in answers.response.answer:
            for rdata in answer:
                if rdata.rdtype == 5:
                    target_ = strip_last_dot(rdata.target.to_text())
                    result.append(['CNAME', host_trg, target_])
                    host_trg = target_

                else:
                    result.append(['A', host_trg, rdata.address])

        return result

    def get_aaaa(self, host_trg):
        """
        Function for resolving the AAAA Record for a given host. Returns an Array of
        the IP Address it resolves to. It will also return CNAME data.
        """
        answers = self.get_answers('AAAA', host_trg)
        if not answers:
            return []

        result = []
        for answer in answers.response.answer:
            for rdata in answer:
                if rdata.rdtype == 5:
                    target_ = strip_last_dot(rdata.target.to_text())
                    result.append(['CNAME', host_trg, target_])
                    host_trg = target_

                else:
                    result.append(['AAAA', host_trg, rdata.address])

        return result

    def get_ip(self, hostname):
        """
        Function resolves a host name to its given A and/or AAAA record.
        Returns Array of found hosts and IPv4 or IPv6 Address.
        """
        found_ip_add = []
        found_ip_add.extend(self.get_a(hostname))
        found_ip_add.extend(self.get_aaaa(hostname))
        return found_ip_add

    def get_mx(self):
        """
        Function for MX Record resolving. Returns all MX records. Returns also the IP
        address of the host both in IPv4 and IPv6. Returns an Array
        """
        answers = self.get_answers('MX', self._domain)
        if not answers:
            return []

        answer_types = ['A', 'AAAA']
        result = []
        for answer_type in answer_types:
            for answer in answers:
                exchange_ = strip_last_dot(answer.exchange.to_text())

                a_or_aaaa_answers = self.get_answers(answer_type, exchange_)
                if not a_or_aaaa_answers:
                    continue

                for a_or_aaaa_answer in a_or_aaaa_answers:
                    result.append(['MX', exchange_, a_or_aaaa_answer.address, answer.preference])

        return result

    def get_ns(self):
        """
        Function for NS Record resolving. Returns all NS records. Returns also the IP
        address of the host both in IPv4 and IPv6. Returns an Array.
        """
        answers = self.get_answers('NS', self._domain)
        if not answers:
            return []

        result = []
        for answer in answers:
            target_ = strip_last_dot(answer.target.to_text())
            addresses = self.get_ip(target_)
            for type_, name_, addr_ in addresses:
                if type_ in ['A', 'AAAA']:
                    result.append(['NS', target_, addr_])

        return result

    def get_soa(self):
        """
        Function for SOA Record resolving. Returns all SOA records. Returns also the IP
        address of the host both in IPv4 and IPv6. Returns an Array.
        """
        queryfunc = dns.query.tcp if self._is_tcp else dns.query.udp

        try:
            querymsg = dns.message.make_query(self._domain, dns.rdatatype.SOA)
            response = queryfunc(querymsg, self._res.nameservers[0], self._res.timeout)
        except (
            OSError,
            dns.exception.Timeout,
            dns.resolver.NXDOMAIN,
            dns.resolver.YXDOMAIN,
            dns.resolver.NoAnswer,
            dns.resolver.NoNameservers,
            dns.query.BadResponse,
        ) as e:
            logger.error(f'Exception "{e}" while resolving SOA record.')
            logger.error(f'Error while resolving SOA while using {self._res.nameservers[0]} as nameserver.')
            return []

        # ~ we consider both response sections
        sections = []
        if len(response.authority) > 0:
            sections.append(response.authority)
        if len(response.answer) > 0:
            sections.append(response.answer)
        else:
            return []

        result = []
        record_types = ['A', 'AAAA']
        for section in sections:
            for record in section:
                if not isinstance(record[0], dns.rdtypes.ANY.SOA.SOA):
                    continue

                mname_ = strip_last_dot(record[0].mname.to_text())

                for record_type in record_types:
                    a_or_aaaa_answers = self.get_answers(record_type, mname_)

                    if not a_or_aaaa_answers:
                        continue

                    for a_or_aaaa_answer in a_or_aaaa_answers:
                        result.append(['SOA', mname_, a_or_aaaa_answer.address])

        return result

    def get_spf(self):
        """
        Function for SPF Record resolving returns the string with the SPF definition.
        Prints the string for the SPF Record and Returns the string
        """
        answers = self.get_answers('SPF', self._domain)
        if not answers:
            return []

        result = []
        for answer in answers:
            strings_ = bytes.join(b'', answer.strings).decode('utf-8', errors='ignore')
            result.append(['SPF', strings_])

        return result

    def get_txt(self, target=None):
        """
        Function for TXT Record resolving returns the string.
        """
        if target is None:
            target = self._domain

        targets = [target, '_dmarc.' + target]
        result = []
        for target_ in targets:
            answers = self.get_answers('TXT', target_)
            if not answers:
                continue

            for answer in answers:
                strings_ = bytes.join(b'', answer.strings).decode('utf-8', errors='ignore')
                result.append(['TXT', target_, strings_])

        return result

    def get_ptr(self, ipaddress):
        """
        Function for resolving PTR Record given it's IPv4 or IPv6 Address.
        """
        reversename_ = dns.reversename.from_address(ipaddress)
        answers = self.get_answers('PTR', reversename_)
        if not answers:
            return []

        result = []
        for answer in answers:
            target_ = strip_last_dot(answer.target.to_text())
            result.append(['PTR', target_, ipaddress])

        return result

    def get_srv(self, host):
        """
        Function for resolving SRV Records.
        """
        answers = self.get_answers('SRV', host)
        if not answers:
            return []

        result = []
        for answer in answers:
            target_ = strip_last_dot(answer.target.to_text())
            a_or_aaaa_answers = self.get_ip(target_)
            for type_, hostname_, addr_ in a_or_aaaa_answers:
                if type_ in ['A', 'AAAA']:
                    result.append(
                        [
                            'SRV',
                            host,
                            target_,
                            addr_,
                            str(answer.port),
                            str(answer.weight),
                        ]
                    )

        return result

    def get_nsec(self, host):
        """
        Function for querying for a NSEC record and retrieving the rdata object.
        This function is used mostly for performing a Zone Walk against a zone.
        """
        return self.get_answers('NSEC', host)

    def from_wire(self, xfr, zone_factory=Zone, relativize=True):
        """
        Method for turning returned data from a DNS AXFR in to RRSET, this method will not perform a
        check origin on the zone data as the method included with dnspython
        """
        z = None
        for r in xfr:
            if z is None:
                if relativize:
                    origin = r.origin
                else:
                    origin = r.answer[0].name
                rdclass = r.answer[0].rdclass
                z = zone_factory(origin, rdclass, relativize=relativize)
            for rrset in r.answer:
                znode = z.nodes.get(rrset.name)
                if not znode:
                    znode = z.node_factory()
                    z.nodes[rrset.name] = znode
                zrds = znode.find_rdataset(rrset.rdclass, rrset.rdtype, rrset.covers, True)
                zrds.update_ttl(rrset.ttl)
                for rd in rrset:
                    try:
                        rd.choose_relativity(z.origin, relativize)
                    except AttributeError:
                        pass
                    zrds.add(rd)

        return z

    def zone_transfer(self):
        """
        Function for testing for zone transfers for a given Domain, it will parse the
        output by record type.
        """
        # if anyone reports a record not parsed, I will add it; the list is long
        # I tried to include those I thought where the most common.

        zone_records = []
        ns_records = []
        logger.info(f'Checking for Zone Transfer for {self._domain} name servers')

        # Find SOA for Domain
        logger.info('Resolving SOA Record')
        try:
            soa_srvs = self.get_soa()
            for type_, name_, addr_ in soa_srvs:
                logger.info(f'\t {type_} {name_} {addr_}')
                ns_records.append(addr_)
        except Exception:
            logger.error('Could not obtain the domains SOA Record.')
            return

        # Find NS for Domain
        logger.info('Resolving NS Records')
        try:
            ns_srvs = []
            ns_srvs = self.get_ns()
            logger.info('NS Servers found:')
            for type_, name_, addr_ in ns_srvs:
                logger.info(f'\t {type_} {name_} {addr_}')
                ns_records.append(addr_)
        except Exception as e:
            logger.error(f'Could not Resolve NS Records: {e}')

        # Remove duplicates
        logger.info('Removing any duplicate NS server IP Addresses...')
        ns_records = list(set(ns_records))

        # Test each NS Server
        for ns_srv in ns_records:
            logger.info(' ')
            logger.info(f'Trying NS server {ns_srv}')

            if not self.check_tcp_dns(ns_srv):
                logger.error(f'Zone Transfer Failed for {ns_srv}!')
                logger.error('Port 53 TCP is being filtered')
                zone_records.append({'type': 'info', 'zone_transfer': 'failed', 'ns_server': ns_srv})
                continue

            logger.info(f'{ns_srv} Has port 53 TCP Open')
            try:
                zone = self.from_wire(dns.query.xfr(ns_srv, self._domain, timeout=DNS_QUERY_TIMEOUT))
                logger.info('Zone Transfer was successful!!')
                zone_records.append({'type': 'info', 'zone_transfer': 'success', 'ns_server': ns_srv})

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.SOA):
                    for rdata in rdataset:
                        mname = strip_last_dot(rdata.mname.to_text())

                        for type_, name_, addr_ in self.get_ip(mname):
                            if type_ in ['A', 'AAAA']:
                                logger.info(f'\t SOA {mname} {addr_}')
                                zone_records.append(
                                    {
                                        'zone_server': ns_srv,
                                        'type': 'SOA',
                                        'mname': mname,
                                        'address': addr_,
                                    }
                                )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.NS):
                    for rdata in rdataset:
                        # Check if target is only the host name or a full FQDN.
                        # If only a hostname we will append the domain name of the
                        # Zone being transfered.
                        target = rdata.target.to_text()
                        if target.count('.') == 0:
                            target = target + '.' + self._domain
                        else:
                            target = strip_last_dot(target)

                        for type_, name_, addr_ in self.get_ip(target):
                            if type_ in ['A', 'AAAA']:
                                logger.info(f'\t NS {target} {addr_}')
                                zone_records.append(
                                    {
                                        'zone_server': ns_srv,
                                        'type': 'NS',
                                        'target': target,
                                        'address': addr_,
                                    }
                                )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.TXT):
                    for rdata in rdataset:
                        s = '; '.join([string.decode() for string in rdata.strings])
                        logger.info(f'\t TXT {s}')
                        zone_records.append({'zone_server': ns_srv, 'type': 'TXT', 'strings': s})

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.SPF):
                    for rdata in rdataset:
                        s = '; '.join([string.decode() for string in rdata.strings])
                        logger.info(f'\t SPF {s}')
                        zone_records.append({'zone_server': ns_srv, 'type': 'SPF', 'strings': s})

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.PTR):
                    for rdata in rdataset:
                        target = rdata.target.to_text() + '.' + self._domain
                        for type_, name_, addr_ in self.get_ip(target):
                            if type_ in ['A', 'AAAA']:
                                logger.info(f'\t PTR {target} {addr_}')
                                zone_records.append(
                                    {
                                        'zone_server': ns_srv,
                                        'type': 'PTR',
                                        'name': target,
                                        'address': addr_,
                                    }
                                )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.MX):
                    for rdata in rdataset:
                        exchange = strip_last_dot(rdata.exchange.to_text())

                        for type_, name_, addr_ in self.get_ip(exchange):
                            fqdn_ = str(name) + '.' + self._domain
                            if type_ in ['A', 'AAAA']:
                                logger.info(f'\t MX {fqdn_} {exchange} {addr_}')
                                zone_records.append(
                                    {
                                        'zone_server': ns_srv,
                                        'type': 'MX',
                                        'name': fqdn_,
                                        'exchange': exchange,
                                        'address': addr_,
                                    }
                                )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.AAAA):
                    fqdn_ = str(name) + '.' + self._domain
                    for rdata in rdataset:
                        logger.info(f'\t AAAA {fqdn_} {rdata.address}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'AAAA',
                                'name': fqdn_,
                                'address': rdata.address,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.A):
                    fqdn_ = str(name) + '.' + self._domain
                    for rdata in rdataset:
                        logger.info(f'\t A {fqdn_} {rdata.address}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'A',
                                'name': fqdn_,
                                'address': rdata.address,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.CNAME):
                    fqdn_ = str(name) + '.' + self._domain
                    for rdata in rdataset:
                        target = strip_last_dot(rdata.target.to_text())

                        for type_, name_, addr_ in self.get_ip(target):
                            if type_ in ['A', 'AAAA']:
                                logger.info(f'\t CNAME {fqdn_} {target} {addr_}')
                                zone_records.append(
                                    {
                                        'zone_server': ns_srv,
                                        'type': 'CNAME',
                                        'name': fqdn_,
                                        'target': target,
                                        'address': addr_,
                                    }
                                )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.SRV):
                    fqdn_ = str(name) + '.' + self._domain

                    for rdata in rdataset:
                        target = strip_last_dot(rdata.target.to_text())
                        weight_ = str(rdata.weight)
                        port_ = str(rdata.port)

                        ip_list = self.get_ip(rdata.target.to_text())
                        if not ip_list:
                            logger.info(f'\t SRV {fqdn_} {target} {port_} {weight_} no_ip')
                            zone_records.append(
                                {
                                    'zone_server': ns_srv,
                                    'type': 'SRV',
                                    'name': fqdn_,
                                    'target': target,
                                    'address': 'no_ip',
                                    'port': port_,
                                    'weight': weight_,
                                }
                            )
                            continue

                        for type_, name_, addr_ in ip_list:
                            if type_ in ['A', 'AAAA']:
                                logger.info(f'\t SRV {fqdn_} {target} {port_} {weight_} {addr_}')
                                zone_records.append(
                                    {
                                        'zone_server': ns_srv,
                                        'type': 'SRV',
                                        'name': fqdn_,
                                        'target': target,
                                        'address': addr_,
                                        'port': port_,
                                        'weight': weight_,
                                    }
                                )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.HINFO):
                    for rdata in rdataset:
                        cpu_ = rdata.cpu.decode()
                        os_ = rdata.os.decode()
                        logger.info(f'\t HINFO {cpu_} {os_}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'HINFO',
                                'cpu': cpu_,
                                'os': os_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.WKS):
                    for rdata in rdataset:
                        addr_ = rdata.address
                        bitmap_ = rdata.bitmap
                        proto_ = rdata.protocol
                        logger.info(f'\t WKS {addr_} {bitmap_} {proto_}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'WKS',
                                'address': addr_,
                                'bitmap': bitmap_,
                                'protocol': proto_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.RP):
                    for rdata in rdataset:
                        mbox_ = rdata.mbox.to_text()
                        txt_ = rdata.txt.to_text()
                        logger.info(f'\t RP {mbox_} {txt_}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'RP',
                                'mbox': mbox_,
                                'txt': txt_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.AFSDB):
                    for rdata in rdataset:
                        subtype_ = str(rdata.subtype)
                        hostname_ = rdata.hostname.to_text()
                        logger.info(f'\t AFSDB {subtype_} {hostname_}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'AFSDB',
                                'subtype': subtype_,
                                'hostname': hostname_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.LOC):
                    for rdata in rdataset:
                        coordinates_ = rdata.to_text()
                        logger.info(f'\t LOC {coordinates_}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'LOC',
                                'coordinates': coordinates_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.X25):
                    for rdata in rdataset:
                        addr_ = rdata.address
                        logger.info(f'\t X25 {addr_}')
                        zone_records.append({'zone_server': ns_srv, 'type': 'X25', 'address': addr_})

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.ISDN):
                    for rdata in rdataset:
                        addr_ = rdata.address
                        logger.info(f'\t ISDN {addr_}')
                        zone_records.append({'zone_server': ns_srv, 'type': 'ISDN', 'address': addr_})

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.RT):
                    for rdata in rdataset:
                        addr_ = rdata.address
                        exchange = strip_last_dot(rdata.exchange.to_text())
                        pref_ = str(rdata.preference)

                        logger.info(f'\t RT {exchange} {pref_}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'X25',
                                'address': addr_,
                                'preference': pref_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.NSAP):
                    for rdata in rdataset:
                        addr_ = rdata.address
                        logger.info(f'\t NSAP {addr_}')
                        zone_records.append({'zone_server': ns_srv, 'type': 'NSAP', 'address': addr_})

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.NAPTR):
                    for rdata in rdataset:
                        flags_ = rdata.flags.decode()
                        order_ = str(rdata.order)
                        pref_ = str(rdata.preference)
                        regexp_ = rdata.regexp.decode()
                        replacement_ = rdata.replacement.to_text()
                        service_ = rdata.service.decode()

                        logger.info(f'\t NAPTR {flags_} {order_} {pref_} {regexp_} {replacement_} {service_}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'NAPTR',
                                'order': order_,
                                'preference': pref_,
                                'regex': regexp_,
                                'replacement': replacement_,
                                'service': service_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.CERT):
                    for rdata in rdataset:
                        algo_ = algorithm_to_text(rdata.algorithm)
                        cert_ = rdata.certificate
                        cert_type_ = rdata.certificate_type
                        key_tag_ = rdata.key_tag

                        logger.info(f'\t CERT {rdata.to_text()}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'CERT',
                                'algorithm': algo_,
                                'certificate': cert_,
                                'certificate_type': cert_type_,
                                'key_tag': key_tag_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.SIG):
                    for rdata in rdataset:
                        algo_ = algorithm_to_text(rdata.algorithm)
                        expiration_ = rdata.expiration
                        inception_ = (rdata.inception,)
                        key_tag_ = rdata.key_tag
                        labels_ = rdata.labels
                        original_ttl_ = rdata.original_ttl
                        signature_ = rdata.signature
                        signer_ = str(rdata.signer)
                        type_covered_ = rdata.type_covered

                        logger.info(
                            f'\t SIG {algo_} {expiration_} {inception_} {key_tag_} {labels_} {original_ttl_} {signature_} {signer_} {type_covered_}'
                        )
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'SIG',
                                'algorithm': algo_,
                                'expiration': expiration_,
                                'inception': inception_,
                                'key_tag': key_tag_,
                                'labels': labels_,
                                'original_ttl': original_ttl_,
                                'signature': signature_,
                                'signer': signer_,
                                'type_covered': type_covered_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.RRSIG):
                    for rdata in rdataset:
                        algo_ = algorithm_to_text(rdata.algorithm)
                        expiration_ = rdata.expiration
                        inception_ = (rdata.inception,)
                        key_tag_ = rdata.key_tag
                        labels_ = rdata.labels
                        original_ttl_ = rdata.original_ttl
                        signature_ = rdata.signature
                        signer_ = str(rdata.signer)
                        type_covered_ = rdata.type_covered

                        logger.info(
                            f'\t RRSIG {algo_} {expiration_} {inception_} {key_tag_} {labels_} {original_ttl_} {signature_} {signer_} {type_covered_}'
                        )
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'RRSIG',
                                'algorithm': algo_,
                                'expiration': expiration_,
                                'inception': inception_,
                                'key_tag': key_tag_,
                                'labels': labels_,
                                'original_ttl': original_ttl_,
                                'signature': signature_,
                                'signer': signer_,
                                'type_covered': type_covered_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.DNSKEY):
                    for rdata in rdataset:
                        algo_ = algorithm_to_text(rdata.algorithm)
                        flags_ = rdata.flags
                        key_ = dns.rdata._hexify(rdata.key)
                        proto_ = rdata.protocol

                        logger.info(f'\t DNSKEY {algo_} {flags_} {key_} {proto_}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'DNSKEY',
                                'algorithm': algo_,
                                'flags': flags_,
                                'key': key_,
                                'protocol': proto_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.DS):
                    for rdata in rdataset:
                        algo_ = algorithm_to_text(rdata.algorithm)
                        digest_ = dns.rdata._hexify(rdata.digest)
                        digest_type_ = rdata.digest_type
                        key_tag_ = rdata.key_tag

                        logger.info(f'\t DS {algo_} {digest_} {digest_type_} {key_tag_}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'DS',
                                'algorithm': algo_,
                                'digest': digest_,
                                'digest_type': digest_type_,
                                'key_tag': key_tag_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.NSEC):
                    for rdata in rdataset:
                        next_ = rdata.next.to_text()
                        logger.info(f'\t NSEC {next_}')
                        zone_records.append({'zone_server': ns_srv, 'type': 'NSEC', 'next': next_})

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.NSEC3):
                    for rdata in rdataset:
                        algo_ = algorithm_to_text(rdata.algorithm)
                        flags_ = rdata.flags
                        iterations_ = rdata.iterations
                        salt_ = dns.rdata._hexify(rdata.salt)

                        logger.info(f'\t NSEC3 {algo_} {flags_} {iterations_} {salt_}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'NSEC3',
                                'algorithm': algo_,
                                'flags': flags_,
                                'iterations': iterations_,
                                'salt': salt_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.NSEC3PARAM):
                    for rdata in rdataset:
                        algo_ = algorithm_to_text(rdata.algorithm)
                        flags_ = rdata.flags
                        iterations_ = rdata.iterations
                        salt_ = dns.rdata._hexify(rdata.salt)

                        logger.info(f'\t NSEC3PARAM {algo_} {flags_} {iterations_} {salt_}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'NSEC3PARAM',
                                'algorithm': algo_,
                                'flags': flags_,
                                'iterations': iterations_,
                                'salt': salt_,
                            }
                        )

                for name, rdataset in zone.iterate_rdatasets(dns.rdatatype.IPSECKEY):
                    for rdata in rdataset:
                        algo_ = algorithm_to_text(rdata.algorithm)
                        key_ = dns.rdata._hexify(rdata.key)
                        gw_ = rdata.gateway
                        gw_type_ = rdata.gateway_type
                        prec_ = rdata.precedence

                        logger.info(f'\t IPSECKEY {algo_} {gw_} {gw_type_} {key_} {prec_}')
                        zone_records.append(
                            {
                                'zone_server': ns_srv,
                                'type': 'IPSECKEY',
                                'algorithm': algo_,
                                'gateway': gw_,
                                'gateway_type': gw_type_,
                                'key': key_,
                                'precedence': prec_,
                            }
                        )
            except Exception as e:
                logger.error(f'Zone Transfer Failed ({e})')
                zone_records.append({'type': 'info', 'zone_transfer': 'failed', 'ns_server': ns_srv})

        return zone_records



#!/usr/bin/python3

# Read the TLD data from https://tld-list.com/df/tld-list-details.json

import argparse
import json


def read_tld_data(file_path):
    with open(file_path) as f:
        return json.load(f)


def filter_and_split_tlds(tld_data, tld_type):
    sponsored = []
    unsponsored = []
    for k, v in tld_data.items():
        if v['type'] == tld_type:
            if v.get('sponsor'):
                sponsored.append(k)
            else:
                unsponsored.append(k)
    return sponsored, unsponsored


def main():
    parser = argparse.ArgumentParser(description='Filter TLD data from a local file')
    parser.add_argument('--file', default='tld-list-details.json', help='Input file containing TLD data')
    parser.add_argument('--type', choices=['gTLD', 'ccTLD'], help='Filter TLDs by type')
    parser.add_argument('--output', default='filtered_tld_list', help='Base name for output files')

    args = parser.parse_args()

    tld_data = read_tld_data(args.file)

    if args.type:
        sponsored, unsponsored = filter_and_split_tlds(tld_data, args.type)
    else:
        sponsored, unsponsored = filter_and_split_tlds(tld_data, None)

    with open(f'{args.output}_sponsored.json', 'w') as f:
        json.dump(sponsored, f, indent=4)

    with open(f'{args.output}_unsponsored.json', 'w') as f:
        json.dump(unsponsored, f, indent=4)

    print(f'Sponsored TLDs have been saved to {args.output}_sponsored.json')
    print(f'Unsponsored TLDs have been saved to {args.output}_unsponsored.json')


if __name__ == '__main__':
    main()



class TLDS:
    @staticmethod
    def generic_tlds():
        gtld_extensions = [
            'aaa',
            'aarp',
            'abarth',
            'abb',
            'abbott',
            'abbvie',
            'abc',
            'able',
            'abogado',
            'abudhabi',
            'academy',
            'accenture',
            'accountant',
            'accountants',
            'aco',
            'active',
            'actor',
            'adac',
            'ads',
            'adult',
            'aeg',
            'aetna',
            'afamilycompany',
            'afl',
            'africa',
            'agakhan',
            'agency',
            'aig',
            'aigo',
            'airbus',
            'airforce',
            'airtel',
            'akdn',
            'alfaromeo',
            'alibaba',
            'alipay',
            'allfinanz',
            'allstate',
            'ally',
            'alsace',
            'alstom',
            'amazon',
            'americanexpress',
            'americanfamily',
            'amex',
            'amfam',
            'amica',
            'amsterdam',
            'analytics',
            'android',
            'anquan',
            'anz',
            'aol',
            'apartments',
            'app',
            'apple',
            'aquarelle',
            'arab',
            'aramco',
            'archi',
            'army',
            'art',
            'arte',
            'asda',
            'associates',
            'athleta',
            'attorney',
            'auction',
            'audi',
            'audible',
            'audio',
            'auspost',
            'author',
            'auto',
            'autos',
            'avianca',
            'aws',
            'axa',
            'azure',
            'baby',
            'baidu',
            'banamex',
            'bananarepublic',
            'band',
            'bank',
            'bar',
            'barcelona',
            'barclaycard',
            'barclays',
            'barefoot',
            'bargains',
            'baseball',
            'basketball',
            'bauhaus',
            'bayern',
            'bbc',
            'bbt',
            'bbva',
            'bcg',
            'bcn',
            'beats',
            'beauty',
            'beer',
            'bentley',
            'berlin',
            'best',
            'bestbuy',
            'bet',
            'bharti',
            'bible',
            'bid',
            'bike',
            'bing',
            'bingo',
            'bio',
            'black',
            'blackfriday',
            'blanco',
            'blockbuster',
            'blog',
            'bloomberg',
            'blue',
            'bms',
            'bmw',
            'bnl',
            'bnpparibas',
            'boats',
            'boehringer',
            'bofa',
            'bom',
            'bond',
            'boo',
            'book',
            'booking',
            'boots',
            'bosch',
            'bostik',
            'boston',
            'bot',
            'boutique',
            'box',
            'bradesco',
            'bridgestone',
            'broadway',
            'broker',
            'brother',
            'brussels',
            'budapest',
            'bugatti',
            'build',
            'builders',
            'business',
            'buy',
            'buzz',
            'bzh',
            'cab',
            'cafe',
            'cal',
            'call',
            'calvinklein',
            'cam',
            'camera',
            'camp',
            'cancerresearch',
            'canon',
            'capetown',
            'capital',
            'capitalone',
            'car',
            'caravan',
            'cards',
            'care',
            'career',
            'careers',
            'cars',
            'cartier',
            'casa',
            'case',
            'caseih',
            'cash',
            'casino',
            'catering',
            'catholic',
            'cba',
            'cbn',
            'cbre',
            'cbs',
            'ceb',
            'center',
            'ceo',
            'cern',
            'cfa',
            'cfd',
            'chanel',
            'channel',
            'charity',
            'chase',
            'chat',
            'cheap',
            'chintai',
            'chloe',
            'christmas',
            'chrome',
            'chrysler',
            'church',
            'cipriani',
            'circle',
            'cisco',
            'citadel',
            'citi',
            'citic',
            'city',
            'cityeats',
            'claims',
            'cleaning',
            'click',
            'clinic',
            'clinique',
            'clothing',
            'cloud',
            'club',
            'clubmed',
            'coach',
            'codes',
            'coffee',
            'college',
            'cologne',
            'com',
            'comcast',
            'commbank',
            'community',
            'company',
            'compare',
            'computer',
            'comsec',
            'condos',
            'construction',
            'consulting',
            'contact',
            'contractors',
            'cooking',
            'cookingchannel',
            'cool',
            'corsica',
            'country',
            'coupon',
            'coupons',
            'courses',
            'cpa',
            'credit',
            'creditcard',
            'creditunion',
            'cricket',
            'crown',
            'crs',
            'cruise',
            'cruises',
            'csc',
            'cuisinella',
            'cymru',
            'cyou',
            'dabur',
            'dad',
            'dance',
            'data',
            'date',
            'dating',
            'datsun',
            'day',
            'dclk',
            'dds',
            'deal',
            'dealer',
            'deals',
            'degree',
            'delivery',
            'dell',
            'deloitte',
            'delta',
            'democrat',
            'dental',
            'dentist',
            'desi',
            'design',
            'dev',
            'dhl',
            'diamonds',
            'diet',
            'digital',
            'direct',
            'directory',
            'discount',
            'discover',
            'dish',
            'diy',
            'dnp',
            'docs',
            'doctor',
            'dodge',
            'dog',
            'doha',
            'domains',
            'doosan',
            'dot',
            'download',
            'drive',
            'dtv',
            'dubai',
            'duck',
            'dunlop',
            'duns',
            'dupont',
            'durban',
            'dvag',
            'dvr',
            'earth',
            'eat',
            'eco',
            'edeka',
            'education',
            'email',
            'emerck',
            'energy',
            'engineer',
            'engineering',
            'enterprises',
            'epost',
            'epson',
            'equipment',
            'ericsson',
            'erni',
            'esq',
            'estate',
            'esurance',
            'etisalat',
            'eurovision',
            'eus',
            'events',
            'everbank',
            'exchange',
            'expert',
            'exposed',
            'express',
            'extraspace',
            'fage',
            'fail',
            'fairwinds',
            'faith',
            'family',
            'fan',
            'fans',
            'farm',
            'farmers',
            'fashion',
            'fast',
            'fedex',
            'feedback',
            'ferrari',
            'ferrero',
            'fiat',
            'fidelity',
            'fido',
            'film',
            'final',
            'finance',
            'financial',
            'fire',
            'firestone',
            'firmdale',
            'fish',
            'fishing',
            'fit',
            'fitness',
            'flickr',
            'flights',
            'flir',
            'florist',
            'flowers',
            'flsmidth',
            'fly',
            'foo',
            'food',
            'foodnetwork',
            'football',
            'ford',
            'forex',
            'forsale',
            'forum',
            'foundation',
            'fox',
            'free',
            'fresenius',
            'frl',
            'frogans',
            'frontdoor',
            'frontier',
            'ftr',
            'fujitsu',
            'fujixerox',
            'fun',
            'fund',
            'furniture',
            'futbol',
            'fyi',
            'gal',
            'gallery',
            'gallo',
            'gallup',
            'game',
            'games',
            'gap',
            'garden',
            'gay',
            'gbiz',
            'gdn',
            'gea',
            'gent',
            'genting',
            'george',
            'ggee',
            'gift',
            'gifts',
            'gives',
            'giving',
            'glade',
            'glass',
            'gle',
            'global',
            'globo',
            'gmail',
            'gmbh',
            'gmo',
            'gmx',
            'godaddy',
            'gold',
            'goldpoint',
            'golf',
            'goo',
            'goodhands',
            'goodyear',
            'goog',
            'google',
            'gop',
            'got',
            'grainger',
            'graphics',
            'gratis',
            'green',
            'gripe',
            'grocery',
            'group',
            'guardian',
            'gucci',
            'guge',
            'guide',
            'guitars',
            'guru',
            'hair',
            'hamburg',
            'hangout',
            'haus',
            'hbo',
            'hdfc',
            'hdfcbank',
            'health',
            'healthcare',
            'help',
            'helsinki',
            'here',
            'hermes',
            'hgtv',
            'hiphop',
            'hisamitsu',
            'hitachi',
            'hiv',
            'hkt',
            'hockey',
            'holdings',
            'holiday',
            'homedepot',
            'homegoods',
            'homes',
            'homesense',
            'honda',
            'honeywell',
            'horse',
            'hospital',
            'host',
            'hosting',
            'hot',
            'hoteles',
            'hotels',
            'hotmail',
            'house',
            'how',
            'hsbc',
            'htc',
            'hughes',
            'hyatt',
            'hyundai',
            'ibm',
            'icbc',
            'ice',
            'icu',
            'ieee',
            'ifm',
            'iinet',
            'ikano',
            'imamat',
            'imdb',
            'immo',
            'immobilien',
            'inc',
            'industries',
            'infiniti',
            'info',
            'ing',
            'ink',
            'institute',
            'insurance',
            'insure',
            'intel',
            'international',
            'intuit',
            'investments',
            'ipiranga',
            'irish',
            'iselect',
            'ismaili',
            'ist',
            'istanbul',
            'itau',
            'itv',
            'iveco',
            'iwc',
            'jaguar',
            'java',
            'jcb',
            'jcp',
            'jeep',
            'jetzt',
            'jewelry',
            'jio',
            'jlc',
            'jll',
            'jmp',
            'jnj',
            'joburg',
            'jot',
            'joy',
            'jpmorgan',
            'jprs',
            'juegos',
            'juniper',
            'kaufen',
            'kddi',
            'kerryhotels',
            'kerrylogistics',
            'kerryproperties',
            'kfh',
            'kia',
            'kids',
            'kim',
            'kinder',
            'kindle',
            'kitchen',
            'kiwi',
            'koeln',
            'komatsu',
            'kosher',
            'kpmg',
            'kpn',
            'krd',
            'kred',
            'kuokgroup',
            'kyoto',
            'lacaixa',
            'ladbrokes',
            'lamborghini',
            'lamer',
            'lancaster',
            'lancia',
            'lancome',
            'land',
            'landrover',
            'lanxess',
            'lasalle',
            'lat',
            'latino',
            'latrobe',
            'law',
            'lawyer',
            'lds',
            'lease',
            'leclerc',
            'lefrak',
            'legal',
            'lego',
            'lexus',
            'lgbt',
            'liaison',
            'lidl',
            'life',
            'lifeinsurance',
            'lifestyle',
            'lighting',
            'like',
            'lilly',
            'limited',
            'limo',
            'lincoln',
            'linde',
            'link',
            'lipsy',
            'live',
            'living',
            'lixil',
            'llc',
            'llp',
            'loan',
            'loans',
            'locker',
            'locus',
            'loft',
            'lol',
            'london',
            'lotte',
            'lotto',
            'love',
            'lpl',
            'lplfinancial',
            'ltd',
            'ltda',
            'lundbeck',
            'lupin',
            'luxe',
            'luxury',
            'macys',
            'madrid',
            'maif',
            'maison',
            'makeup',
            'man',
            'management',
            'mango',
            'map',
            'market',
            'marketing',
            'markets',
            'marriott',
            'marshalls',
            'maserati',
            'mattel',
            'mba',
            'mcd',
            'mcdonalds',
            'mckinsey',
            'med',
            'media',
            'meet',
            'melbourne',
            'meme',
            'memorial',
            'men',
            'menu',
            'meo',
            'merckmsd',
            'metlife',
            'miami',
            'microsoft',
            'mini',
            'mint',
            'mit',
            'mitsubishi',
            'mlb',
            'mls',
            'mma',
            'mobi',
            'mobile',
            'mobily',
            'moda',
            'moe',
            'moi',
            'mom',
            'monash',
            'money',
            'monster',
            'montblanc',
            'mopar',
            'mormon',
            'mortgage',
            'moscow',
            'moto',
            'motorcycles',
            'mov',
            'movie',
            'movistar',
            'msd',
            'mtn',
            'mtpc',
            'mtr',
            'music',
            'mutual',
            'mutuelle',
            'nab',
            'nadex',
            'nagoya',
            'nationwide',
            'natura',
            'navy',
            'nba',
            'nec',
            'net',
            'netbank',
            'netflix',
            'network',
            'neustar',
            'new',
            'newholland',
            'news',
            'next',
            'nextdirect',
            'nexus',
            'nfl',
            'ngo',
            'nhk',
            'nico',
            'nike',
            'nikon',
            'ninja',
            'nissan',
            'nissay',
            'nokia',
            'northwesternmutual',
            'norton',
            'now',
            'nowruz',
            'nowtv',
            'nra',
            'nrw',
            'ntt',
            'nyc',
            'obi',
            'observer',
            'off',
            'office',
            'okinawa',
            'olayan',
            'olayangroup',
            'oldnavy',
            'ollo',
            'omega',
            'one',
            'ong',
            'onl',
            'online',
            'onyourside',
            'ooo',
            'open',
            'oracle',
            'orange',
            'org',
            'organic',
            'orientexpress',
            'origins',
            'osaka',
            'otsuka',
            'ott',
            'ovh',
            'page',
            'pamperedchef',
            'panasonic',
            'panerai',
            'paris',
            'pars',
            'partners',
            'parts',
            'party',
            'passagens',
            'pay',
            'pccw',
            'pet',
            'pfizer',
            'pharmacy',
            'phd',
            'philips',
            'phone',
            'photo',
            'photography',
            'photos',
            'physio',
            'piaget',
            'pics',
            'pictet',
            'pictures',
            'pid',
            'pin',
            'ping',
            'pink',
            'pioneer',
            'pizza',
            'place',
            'play',
            'playstation',
            'plumbing',
            'plus',
            'pnc',
            'pohl',
            'poker',
            'politie',
            'porn',
            'pramerica',
            'praxi',
            'press',
            'prime',
            'prod',
            'productions',
            'prof',
            'progressive',
            'promo',
            'properties',
            'property',
            'protection',
            'pru',
            'prudential',
            'pub',
            'pwc',
            'qpon',
            'quebec',
            'quest',
            'qvc',
            'racing',
            'radio',
            'raid',
            'read',
            'realestate',
            'realtor',
            'realty',
            'recipes',
            'red',
            'redstone',
            'redumbrella',
            'rehab',
            'reise',
            'reisen',
            'reit',
            'reliance',
            'ren',
            'rent',
            'rentals',
            'repair',
            'report',
            'republican',
            'rest',
            'restaurant',
            'review',
            'reviews',
            'rexroth',
            'rich',
            'richardli',
            'ricoh',
            'rightathome',
            'ril',
            'rio',
            'rip',
            'rmit',
            'rocher',
            'rocks',
            'rodeo',
            'rogers',
            'room',
            'rsvp',
            'rugby',
            'ruhr',
            'run',
            'rwe',
            'ryukyu',
            'saarland',
            'safe',
            'safety',
            'sakura',
            'sale',
            'salon',
            'samsclub',
            'samsung',
            'sandvik',
            'sandvikcoromant',
            'sanofi',
            'sap',
            'sapo',
            'sarl',
            'sas',
            'save',
            'saxo',
            'sbi',
            'sbs',
            'sca',
            'scb',
            'schaeffler',
            'schmidt',
            'scholarships',
            'school',
            'schule',
            'schwarz',
            'science',
            'scjohnson',
            'scor',
            'scot',
            'search',
            'seat',
            'secure',
            'security',
            'seek',
            'select',
            'sener',
            'services',
            'ses',
            'seven',
            'sew',
            'sex',
            'sexy',
            'sfr',
            'shangrila',
            'sharp',
            'shaw',
            'shell',
            'shia',
            'shiksha',
            'shoes',
            'shop',
            'shopping',
            'shouji',
            'show',
            'showtime',
            'shriram',
            'silk',
            'sina',
            'singles',
            'site',
            'ski',
            'skin',
            'sky',
            'skype',
            'sling',
            'smart',
            'smile',
            'sncf',
            'soccer',
            'social',
            'softbank',
            'software',
            'sohu',
            'solar',
            'solutions',
            'song',
            'sony',
            'soy',
            'spa',
            'space',
            'spiegel',
            'sport',
            'spot',
            'spreadbetting',
            'srl',
            'srt',
            'stada',
            'staples',
            'star',
            'starhub',
            'statebank',
            'statefarm',
            'statoil',
            'stc',
            'stcgroup',
            'stockholm',
            'storage',
            'store',
            'stream',
            'studio',
            'study',
            'style',
            'sucks',
            'supplies',
            'supply',
            'support',
            'surf',
            'surgery',
            'suzuki',
            'swatch',
            'swiftcover',
            'swiss',
            'sydney',
            'symantec',
            'systems',
            'tab',
            'taipei',
            'talk',
            'taobao',
            'target',
            'tatamotors',
            'tatar',
            'tattoo',
            'tax',
            'taxi',
            'tci',
            'tdk',
            'team',
            'tech',
            'technology',
            'telecity',
            'telefonica',
            'temasek',
            'tennis',
            'teva',
            'thd',
            'theater',
            'theatre',
            'tiaa',
            'tickets',
            'tienda',
            'tiffany',
            'tips',
            'tires',
            'tirol',
            'tjmaxx',
            'tjx',
            'tkmaxx',
            'tmall',
            'today',
            'tokyo',
            'tools',
            'top',
            'toray',
            'toshiba',
            'total',
            'tours',
            'town',
            'toyota',
            'toys',
            'trade',
            'trading',
            'training',
            'travelchannel',
            'travelers',
            'travelersinsurance',
            'trust',
            'trv',
            'tube',
            'tui',
            'tunes',
            'tushu',
            'tvs',
            'ubank',
            'ubs',
            'uconnect',
            'unicom',
            'university',
            'uno',
            'uol',
            'ups',
            'vacations',
            'vana',
            'vanguard',
            'vegas',
            'ventures',
            'verisign',
            'versicherung',
            'vet',
            'viajes',
            'video',
            'vig',
            'viking',
            'villas',
            'vin',
            'vip',
            'virgin',
            'visa',
            'vision',
            'vista',
            'vistaprint',
            'viva',
            'vivo',
            'vlaanderen',
            'vodka',
            'volkswagen',
            'volvo',
            'vote',
            'voting',
            'voto',
            'voyage',
            'vuelos',
            'wales',
            'walmart',
            'walter',
            'wang',
            'wanggou',
            'warman',
            'watch',
            'watches',
            'weather',
            'weatherchannel',
            'webcam',
            'weber',
            'website',
            'wed',
            'wedding',
            'weibo',
            'weir',
            'whoswho',
            'wien',
            'wiki',
            'williamhill',
            'win',
            'windows',
            'wine',
            'winners',
            'wme',
            'wolterskluwer',
            'woodside',
            'work',
            'works',
            'world',
            'wow',
            'wtc',
            'wtf',
            'xbox',
            'xerox',
            'xfinity',
            'xihuan',
            'xin',
            'xperia',
            'xyz',
            'yachts',
            'yahoo',
            'yamaxun',
            'yandex',
            'yodobashi',
            'yoga',
            'yokohama',
            'you',
            'youtube',
            'yun',
            'zappos',
            'zara',
            'zero',
            'zip',
            'zippo',
            'zone',
            'zuerich',
        ]
        return gtld_extensions

    @staticmethod
    def country_codes():
        country_extensions = [
            'ac',
            'ad',
            'ae',
            'af',
            'ag',
            'ai',
            'al',
            'am',
            'an',
            'ao',
            'aq',
            'ar',
            'as',
            'at',
            'au',
            'aw',
            'ax',
            'az',
            'ba',
            'bb',
            'bd',
            'be',
            'bf',
            'bg',
            'bh',
            'bi',
            'bj',
            'bl',
            'bm',
            'bn',
            'bo',
            'bq',
            'br',
            'bs',
            'bt',
            'bv',
            'bw',
            'by',
            'bz',
            'ca',
            'cc',
            'cd',
            'cf',
            'cg',
            'ch',
            'ci',
            'ck',
            'cl',
            'cm',
            'cn',
            'co',
            'cr',
            'cu',
            'cv',
            'cw',
            'cx',
            'cy',
            'cz',
            'de',
            'dj',
            'dk',
            'dm',
            'do',
            'dz',
            'ec',
            'ee',
            'eg',
            'eh',
            'er',
            'es',
            'et',
            'eu',
            'fi',
            'fj',
            'fk',
            'fm',
            'fo',
            'fr',
            'ga',
            'gb',
            'gd',
            'ge',
            'gf',
            'gg',
            'gh',
            'gi',
            'gl',
            'gm',
            'gn',
            'gp',
            'gq',
            'gr',
            'gs',
            'gt',
            'gu',
            'gw',
            'gy',
            'hk',
            'hm',
            'hn',
            'hr',
            'ht',
            'hu',
            'id',
            'ie',
            'il',
            'im',
            'in',
            'io',
            'iq',
            'ir',
            'is',
            'it',
            'je',
            'jm',
            'jo',
            'jp',
            'ke',
            'kg',
            'kh',
            'ki',
            'km',
            'kn',
            'kp',
            'kr',
            'kw',
            'ky',
            'kz',
            'la',
            'lb',
            'lc',
            'li',
            'lk',
            'lr',
            'ls',
            'lt',
            'lu',
            'lv',
            'ly',
            'ma',
            'mc',
            'md',
            'me',
            'mf',
            'mg',
            'mh',
            'mk',
            'ml',
            'mm',
            'mn',
            'mo',
            'mp',
            'mq',
            'mr',
            'ms',
            'mt',
            'mu',
            'mv',
            'mw',
            'mx',
            'my',
            'mz',
            'na',
            'nc',
            'ne',
            'nf',
            'ng',
            'ni',
            'nl',
            'no',
            'np',
            'nr',
            'nu',
            'nz',
            'om',
            'pa',
            'pe',
            'pf',
            'pg',
            'ph',
            'pk',
            'pl',
            'pm',
            'pn',
            'pr',
            'ps',
            'pt',
            'pw',
            'py',
            'qa',
            're',
            'ro',
            'rs',
            'ru',
            'rw',
            'sa',
            'sb',
            'sc',
            'sd',
            'se',
            'sg',
            'sh',
            'si',
            'sj',
            'sk',
            'sl',
            'sm',
            'sn',
            'so',
            'sr',
            'ss',
            'st',
            'su',
            'sv',
            'sx',
            'sy',
            'sz',
            'tc',
            'td',
            'tf',
            'tg',
            'th',
            'tj',
            'tk',
            'tl',
            'tm',
            'tn',
            'to',
            'tp',
            'tr',
            'tt',
            'tv',
            'tw',
            'tz',
            'ua',
            'ug',
            'uk',
            'um' 'us',
            'uy',
            'uz',
            'va',
            'vc',
            've',
            'vg',
            'vi',
            'vn',
            'vu',
            'wf',
            'ws',
            'ye',
            'yt',
            'za',
            'zm',
            'zw',
        ]
        return country_extensions

    @staticmethod
    def sponsored_tlds():
        sponsored_extensions = [
            'aaa',
            'aarp',
            'abb',
            'abbott',
            'abbvie',
            'abc',
            'able',
            'abogado',
            'abudhabi',
            'academy',
            'accenture',
            'accountant',
            'accountants',
            'aco',
            'actor',
            'ads',
            'adult',
            'aeg',
            'aetna',
            'afl',
            'africa',
            'agakhan',
            'agency',
            'aig',
            'airbus',
            'airforce',
            'airtel',
            'akdn',
            'alibaba',
            'alipay',
            'allfinanz',
            'allstate',
            'ally',
            'alsace',
            'alstom',
            'amazon',
            'americanexpress',
            'americanfamily',
            'amex',
            'amfam',
            'amica',
            'amsterdam',
            'analytics',
            'android',
            'anquan',
            'anz',
            'aol',
            'apartments',
            'app',
            'apple',
            'aquarelle',
            'arab',
            'aramco',
            'archi',
            'army',
            'art',
            'arte',
            'asda',
            'associates',
            'athleta',
            'attorney',
            'auction',
            'audi',
            'audible',
            'audio',
            'auspost',
            'author',
            'auto',
            'autos',
            'aws',
            'axa',
            'azure',
            'baby',
            'baidu',
            'banamex',
            'band',
            'bank',
            'bar',
            'barcelona',
            'barclaycard',
            'barclays',
            'barefoot',
            'bargains',
            'baseball',
            'basketball',
            'bauhaus',
            'bayern',
            'bbc',
            'bbt',
            'bbva',
            'bcg',
            'bcn',
            'beats',
            'beauty',
            'beer',
            'bentley',
            'berlin',
            'best',
            'bestbuy',
            'bet',
            'bharti',
            'bible',
            'bid',
            'bike',
            'bing',
            'bingo',
            'bio',
            'black',
            'blackfriday',
            'blockbuster',
            'blog',
            'bloomberg',
            'blue',
            'bms',
            'bmw',
            'bnpparibas',
            'boats',
            'boehringer',
            'bofa',
            'bom',
            'bond',
            'boo',
            'book',
            'booking',
            'bosch',
            'bostik',
            'boston',
            'bot',
            'boutique',
            'box',
            'bradesco',
            'bridgestone',
            'broadway',
            'broker',
            'brother',
            'brussels',
            'build',
            'builders',
            'business',
            'buy',
            'buzz',
            'bzh',
            'cab',
            'cafe',
            'cal',
            'call',
            'calvinklein',
            'cam',
            'camera',
            'camp',
            'canon',
            'capetown',
            'capital',
            'capitalone',
            'car',
            'caravan',
            'cards',
            'care',
            'career',
            'careers',
            'cars',
            'casa',
            'case',
            'cash',
            'casino',
            'catering',
            'catholic',
            'cba',
            'cbn',
            'cbre',
            'center',
            'ceo',
            'cern',
            'cfa',
            'cfd',
            'chanel',
            'channel',
            'charity',
            'chase',
            'chat',
            'cheap',
            'chintai',
            'christmas',
            'chrome',
            'church',
            'cipriani',
            'circle',
            'cisco',
            'citadel',
            'citi',
            'citic',
            'city',
            'claims',
            'cleaning',
            'click',
            'clinic',
            'clinique',
            'clothing',
            'cloud',
            'club',
            'clubmed',
            'coach',
            'codes',
            'coffee',
            'college',
            'cologne',
            'com',
            'commbank',
            'community',
            'company',
            'compare',
            'computer',
            'comsec',
            'condos',
            'construction',
            'consulting',
            'contact',
            'contractors',
            'cooking',
            'cool',
            'corsica',
            'country',
            'coupon',
            'coupons',
            'courses',
            'cpa',
            'credit',
            'creditcard',
            'creditunion',
            'cricket',
            'crown',
            'crs',
            'cruise',
            'cruises',
            'cuisinella',
            'cymru',
            'cyou',
            'dad',
            'dance',
            'data',
            'date',
            'dating',
            'datsun',
            'day',
            'dclk',
            'dds',
            'deal',
            'dealer',
            'deals',
            'degree',
            'delivery',
            'dell',
            'deloitte',
            'delta',
            'democrat',
            'dental',
            'dentist',
            'desi',
            'design',
            'dev',
            'dhl',
            'diamonds',
            'diet',
            'digital',
            'direct',
            'directory',
            'discount',
            'discover',
            'dish',
            'diy',
            'dnp',
            'docs',
            'doctor',
            'dog',
            'domains',
            'dot',
            'download',
            'drive',
            'dtv',
            'dubai',
            'dunlop',
            'dupont',
            'durban',
            'dvag',
            'dvr',
            'earth',
            'eat',
            'eco',
            'edeka',
            'education',
            'email',
            'emerck',
            'energy',
            'engineer',
            'engineering',
            'enterprises',
            'epson',
            'equipment',
            'ericsson',
            'erni',
            'esq',
            'estate',
            'eurovision',
            'eus',
            'events',
            'exchange',
            'expert',
            'exposed',
            'express',
            'extraspace',
            'fage',
            'fail',
            'fairwinds',
            'faith',
            'family',
            'fan',
            'fans',
            'farm',
            'farmers',
            'fashion',
            'fast',
            'fedex',
            'feedback',
            'ferrari',
            'ferrero',
            'fidelity',
            'fido',
            'film',
            'final',
            'finance',
            'financial',
            'fire',
            'firestone',
            'firmdale',
            'fish',
            'fishing',
            'fit',
            'fitness',
            'flickr',
            'flights',
            'flir',
            'florist',
            'flowers',
            'fly',
            'foo',
            'food',
            'football',
            'ford',
            'forex',
            'forsale',
            'forum',
            'foundation',
            'fox',
            'free',
            'fresenius',
            'frl',
            'frogans',
            'frontier',
            'ftr',
            'fujitsu',
            'fun',
            'fund',
            'furniture',
            'futbol',
            'fyi',
            'gal',
            'gallery',
            'gallo',
            'gallup',
            'game',
            'games',
            'gap',
            'garden',
            'gay',
            'gbiz',
            'gdn',
            'gea',
            'gent',
            'genting',
            'george',
            'ggee',
            'gift',
            'gifts',
            'gives',
            'giving',
            'glass',
            'gle',
            'global',
            'globo',
            'gmail',
            'gmbh',
            'gmo',
            'gmx',
            'godaddy',
            'gold',
            'goldpoint',
            'golf',
            'goo',
            'goodyear',
            'goog',
            'google',
            'gop',
            'got',
            'grainger',
            'graphics',
            'gratis',
            'green',
            'gripe',
            'grocery',
            'group',
            'gucci',
            'guge',
            'guide',
            'guitars',
            'guru',
            'hair',
            'hamburg',
            'hangout',
            'haus',
            'hbo',
            'hdfc',
            'hdfcbank',
            'health',
            'healthcare',
            'help',
            'helsinki',
            'here',
            'hermes',
            'hiphop',
            'hisamitsu',
            'hitachi',
            'hiv',
            'hkt',
            'hockey',
            'holdings',
            'holiday',
            'homedepot',
            'homegoods',
            'homes',
            'homesense',
            'honda',
            'horse',
            'hospital',
            'host',
            'hosting',
            'hot',
            'hotels',
            'hotmail',
            'house',
            'how',
            'hsbc',
            'hughes',
            'hyatt',
            'hyundai',
            'ibm',
            'icbc',
            'ice',
            'icu',
            'ieee',
            'ifm',
            'ikano',
            'imamat',
            'imdb',
            'immo',
            'immobilien',
            'inc',
            'industries',
            'infiniti',
            'info',
            'ing',
            'ink',
            'institute',
            'insurance',
            'insure',
            'international',
            'intuit',
            'investments',
            'ipiranga',
            'irish',
            'ismaili',
            'ist',
            'istanbul',
            'itau',
            'itv',
            'jaguar',
            'java',
            'jcb',
            'jeep',
            'jetzt',
            'jewelry',
            'jio',
            'jll',
            'jmp',
            'jnj',
            'joburg',
            'jot',
            'joy',
            'jpmorgan',
            'jprs',
            'juegos',
            'juniper',
            'kaufen',
            'kddi',
            'kerryhotels',
            'kerrylogistics',
            'kerryproperties',
            'kfh',
            'kia',
            'kids',
            'kim',
            'kindle',
            'kitchen',
            'kiwi',
            'koeln',
            'komatsu',
            'kosher',
            'kpmg',
            'kpn',
            'krd',
            'kred',
            'kuokgroup',
            'kyoto',
            'lacaixa',
            'lamborghini',
            'lamer',
            'lancaster',
            'land',
            'landrover',
            'lanxess',
            'lasalle',
            'lat',
            'latino',
            'latrobe',
            'law',
            'lawyer',
            'lds',
            'lease',
            'leclerc',
            'lefrak',
            'legal',
            'lego',
            'lexus',
            'lgbt',
            'lidl',
            'life',
            'lifeinsurance',
            'lifestyle',
            'lighting',
            'like',
            'lilly',
            'limited',
            'limo',
            'lincoln',
            'link',
            'lipsy',
            'live',
            'living',
            'llc',
            'llp',
            'loan',
            'loans',
            'locker',
            'locus',
            'lol',
            'london',
            'lotte',
            'lotto',
            'love',
            'lpl',
            'lplfinancial',
            'ltd',
            'ltda',
            'lundbeck',
            'luxe',
            'luxury',
            'madrid',
            'maif',
            'maison',
            'makeup',
            'man',
            'management',
            'mango',
            'map',
            'market',
            'marketing',
            'markets',
            'marriott',
            'marshalls',
            'mattel',
            'mba',
            'mckinsey',
            'med',
            'media',
            'meet',
            'melbourne',
            'meme',
            'memorial',
            'men',
            'menu',
            'merckmsd',
            'miami',
            'microsoft',
            'mini',
            'mint',
            'mit',
            'mitsubishi',
            'mlb',
            'mls',
            'mma',
            'mobi',
            'mobile',
            'moda',
            'moe',
            'moi',
            'mom',
            'monash',
            'money',
            'monster',
            'mormon',
            'mortgage',
            'moscow',
            'moto',
            'motorcycles',
            'mov',
            'movie',
            'msd',
            'mtn',
            'mtr',
            'music',
            'nab',
            'nagoya',
            'navy',
            'nba',
            'nec',
            'net',
            'netbank',
            'netflix',
            'network',
            'neustar',
            'new',
            'news',
            'next',
            'nextdirect',
            'nexus',
            'nfl',
            'ngo',
            'nhk',
            'nico',
            'nike',
            'nikon',
            'ninja',
            'nissan',
            'nissay',
            'nokia',
            'norton',
            'now',
            'nowruz',
            'nowtv',
            'nra',
            'nrw',
            'ntt',
            'nyc',
            'obi',
            'observer',
            'office',
            'okinawa',
            'olayan',
            'olayangroup',
            'ollo',
            'omega',
            'one',
            'ong',
            'onl',
            'online',
            'ooo',
            'open',
            'oracle',
            'orange',
            'org',
            'organic',
            'origins',
            'osaka',
            'otsuka',
            'ott',
            'ovh',
            'page',
            'panasonic',
            'paris',
            'pars',
            'partners',
            'parts',
            'party',
            'pay',
            'pccw',
            'pet',
            'pfizer',
            'pharmacy',
            'phd',
            'philips',
            'phone',
            'photo',
            'photography',
            'photos',
            'physio',
            'pics',
            'pictet',
            'pictures',
            'pid',
            'pin',
            'ping',
            'pink',
            'pioneer',
            'pizza',
            'place',
            'play',
            'playstation',
            'plumbing',
            'plus',
            'pnc',
            'pohl',
            'poker',
            'politie',
            'porn',
            'pramerica',
            'praxi',
            'press',
            'prime',
            'prod',
            'productions',
            'prof',
            'progressive',
            'promo',
            'properties',
            'property',
            'protection',
            'pru',
            'prudential',
            'pub',
            'pwc',
            'qpon',
            'quebec',
            'quest',
            'racing',
            'radio',
            'read',
            'realestate',
            'realtor',
            'realty',
            'recipes',
            'red',
            'redstone',
            'redumbrella',
            'rehab',
            'reise',
            'reisen',
            'reit',
            'reliance',
            'ren',
            'rent',
            'rentals',
            'repair',
            'report',
            'republican',
            'rest',
            'restaurant',
            'review',
            'reviews',
            'rexroth',
            'rich',
            'richardli',
            'ricoh',
            'ril',
            'rio',
            'rip',
            'rocks',
            'rodeo',
            'rogers',
            'room',
            'rsvp',
            'rugby',
            'ruhr',
            'run',
            'rwe',
            'ryukyu',
            'saarland',
            'safe',
            'safety',
            'sakura',
            'sale',
            'salon',
            'samsclub',
            'samsung',
            'sandvik',
            'sandvikcoromant',
            'sanofi',
            'sap',
            'sarl',
            'sas',
            'save',
            'saxo',
            'sbi',
            'sbs',
            'scb',
            'schaeffler',
            'schmidt',
            'scholarships',
            'school',
            'schule',
            'schwarz',
            'science',
            'scot',
            'search',
            'seat',
            'secure',
            'security',
            'seek',
            'select',
            'sener',
            'services',
            'seven',
            'sew',
            'sex',
            'sexy',
            'sfr',
            'shangrila',
            'sharp',
            'shell',
            'shia',
            'shiksha',
            'shoes',
            'shop',
            'shopping',
            'shouji',
            'show',
            'silk',
            'sina',
            'singles',
            'site',
            'ski',
            'skin',
            'sky',
            'skype',
            'sling',
            'smart',
            'smile',
            'sncf',
            'soccer',
            'social',
            'softbank',
            'software',
            'sohu',
            'solar',
            'solutions',
            'song',
            'sony',
            'soy',
            'spa',
            'space',
            'sport',
            'spot',
            'srl',
            'stada',
            'staples',
            'star',
            'statebank',
            'statefarm',
            'stc',
            'stcgroup',
            'stockholm',
            'storage',
            'store',
            'stream',
            'studio',
            'study',
            'style',
            'sucks',
            'supplies',
            'supply',
            'support',
            'surf',
            'surgery',
            'suzuki',
            'swatch',
            'swiss',
            'sydney',
            'systems',
            'tab',
            'taipei',
            'talk',
            'taobao',
            'target',
            'tatamotors',
            'tatar',
            'tattoo',
            'tax',
            'taxi',
            'tci',
            'tdk',
            'team',
            'tech',
            'technology',
            'temasek',
            'tennis',
            'teva',
            'thd',
            'theater',
            'theatre',
            'tiaa',
            'tickets',
            'tienda',
            'tips',
            'tires',
            'tirol',
            'tjmaxx',
            'tjx',
            'tkmaxx',
            'tmall',
            'today',
            'tokyo',
            'tools',
            'top',
            'toray',
            'toshiba',
            'total',
            'tours',
            'town',
            'toyota',
            'toys',
            'trade',
            'trading',
            'training',
            'travelers',
            'travelersinsurance',
            'trust',
            'trv',
            'tube',
            'tui',
            'tunes',
            'tushu',
            'tvs',
            'ubank',
            'ubs',
            'unicom',
            'university',
            'uno',
            'uol',
            'ups',
            'vacations',
            'vana',
            'vanguard',
            'vegas',
            'ventures',
            'verisign',
            'versicherung',
            'vet',
            'viajes',
            'video',
            'vig',
            'viking',
            'villas',
            'vin',
            'vip',
            'virgin',
            'visa',
            'vision',
            'viva',
            'vivo',
            'vlaanderen',
            'vodka',
            'volvo',
            'vote',
            'voting',
            'voto',
            'voyage',
            'wales',
            'walmart',
            'walter',
            'wang',
            'wanggou',
            'watch',
            'watches',
            'weather',
            'weatherchannel',
            'webcam',
            'weber',
            'website',
            'wed',
            'wedding',
            'weibo',
            'weir',
            'whoswho',
            'wien',
            'wiki',
            'williamhill',
            'win',
            'windows',
            'wine',
            'winners',
            'wme',
            'wolterskluwer',
            'woodside',
            'work',
            'works',
            'world',
            'wow',
            'wtc',
            'wtf',
            'xbox',
            'xerox',
            'xihuan',
            'xin',
            'xyz',
            'yachts',
            'yahoo',
            'yamaxun',
            'yandex',
            'yodobashi',
            'yoga',
            'yokohama',
            'you',
            'youtube',
            'yun',
            'zappos',
            'zara',
            'zero',
            'zip',
            'zone',
            'zuerich',
        ]
        return sponsored_extensions



#!/usr/bin/env python3

#    DNSRecon Data Parser
#
#    Copyright (C) 2012  Carlos Perez
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; Applies version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

__version__ = '0.0.7'
__author__ = 'Carlos Perez, Carlos_Perez@darkoperator.com'

import csv
import getopt
import os
import re
import sys
import xml.etree.ElementTree as cElementTree

from netaddr import *

# Function Definitions
# ------------------------------------------------------------------------------


def print_status(message=''):
    print(f'\033[1;34m[*]\033[1;m {message}')


def print_good(message=''):
    print(f'\033[1;32m[*]\033[1;m {message}')


def print_error(message=''):
    print(f'\033[1;31m[-]\033[1;m {message}')


def print_debug(message=''):
    print(f'\033[1;31m[!]\033[1;m {message}')


def print_line(message=''):
    print(f'{message}')


def process_range(arg):
    """
    This function will take a string representation of a range for IPv4 or IPv6 in
    CIDR or Range format and return a list of IPs.
    """
    try:
        ip_list = None
        range_vals = []
        if re.match(r'\S*/\S*', arg):
            ip_list = IPNetwork(arg)

        range_vals.extend(arg.split('-'))
        if len(range_vals) == 2:
            ip_list = IPNetwork(IPRange(range_vals[0], range_vals[1])).cidrs()[-1]
    except Exception:
        print_error(f'Range provided is not valid: {arg()}')
        return []
    return ip_list


def xml_parse(xm_file, ifilter, tfilter, nfilter, list):
    """
    Function for parsing XML files created by DNSRecon and apply filters.
    """
    iplist = []
    for event, elem in cElementTree.iterparse(xm_file):
        # Check if it is a record
        if elem.tag == 'record':
            # Check that it is a RR Type that has an IP Address
            if 'address' in elem.attrib:
                # Check if the IP is in the filter list of IPs to ignore
                if (len(ifilter) == 0 or IPAddress(elem.attrib['address']) in ifilter) and (elem.attrib['address'] != 'no_ip'):
                    # Check if the RR Type against the types
                    if re.match(tfilter, elem.attrib['type'], re.I):
                        # Process A, AAAA and PTR Records
                        if re.search(r'PTR|^[A]$|AAAA', elem.attrib['type']) and re.search(nfilter, elem.attrib['name'], re.I):
                            if list:
                                if elem.attrib['address'] not in iplist:
                                    print(elem.attrib['address'])
                            else:
                                print_good(f"{elem.attrib['type']} {elem.attrib['name']} {elem.attrib['address']}")

                        # Process NS Records
                        elif re.search(r'NS', elem.attrib['type']) and re.search(nfilter, elem.attrib['target'], re.I):
                            if list:
                                if elem.attrib['address'] not in iplist:
                                    iplist.append(elem.attrib['address'])
                            else:
                                print_good(f"{elem.attrib['type']} {elem.attrib['target']} {elem.attrib['address']}")

                        # Process SOA Records
                        elif re.search(r'SOA', elem.attrib['type']) and re.search(nfilter, elem.attrib['mname'], re.I):
                            if list:
                                if elem.attrib['address'] not in iplist:
                                    iplist.append(elem.attrib['address'])
                            else:
                                print_good(f"{elem.attrib['type']} {elem.attrib['mname']} {elem.attrib['address']}")

                        # Process MS Records
                        elif re.search(r'MX', elem.attrib['type']) and re.search(nfilter, elem.attrib['exchange'], re.I):
                            if list:
                                if elem.attrib['address'] not in iplist:
                                    iplist.append(elem.attrib['address'])
                            else:
                                print_good(f"{elem.attrib['type']} {elem.attrib['exchange']} {elem.attrib['address']}")

                        # Process SRV Records
                        elif re.search(r'SRV', elem.attrib['type']) and re.search(nfilter, elem.attrib['target'], re.I):
                            if list:
                                if elem.attrib['address'] not in iplist:
                                    iplist.append(elem.attrib['address'])
                            else:
                                print_good(
                                    '{0} {1} {2} {3} {4}'.format(
                                        elem.attrib['type'],
                                        elem.attrib['name'],
                                        elem.attrib['address'],
                                        elem.attrib['target'],
                                        elem.attrib['port'],
                                    )
                                )
            else:
                if re.match(tfilter, elem.attrib['type'], re.I):
                    # Process TXT and SPF Records
                    if re.search(r'TXT|SPF', elem.attrib['type']):
                        if not list:
                            print_good('{0} {1}'.format(elem.attrib['type'], elem.attrib['strings']))
    # Process IPs in a list
    if len(iplist) > 0:
        try:
            for ip in filter(None, iplist):
                print_line(ip)
        except OSError:
            sys.exit(0)


def csv_parse(csv_file, ifilter, tfilter, nfilter, list):
    """
    Function for parsing CSV files created by DNSRecon and apply filters.
    """
    iplist = []
    reader = csv.reader(open(csv_file), delimiter=',')
    next(reader)
    for row in reader:
        # Check if IP is in the filter list of addresses to ignore
        if ((len(ifilter) == 0) or (IPAddress(row[2]) in ifilter)) and (row[2] != 'no_ip'):
            # Check Host Name regex and type list
            if re.search(tfilter, row[0], re.I) and re.search(nfilter, row[1], re.I):
                if list:
                    if row[2] not in iplist:
                        print(row[2])
                else:
                    print_good(' '.join(row))
    # Process IPs for target list if available
    # if len(iplist) > 0:
    #    for ip in filter(None, iplist):
    #        print_line(ip)


def extract_hostnames(file):
    host_names = []
    hostname_pattern = re.compile('(^[^.]*)')
    file_type = detect_type(file)
    if file_type == 'xml':
        for event, elem in cElementTree.iterparse(file):
            # Check if it is a record
            if elem.tag == 'record':
                # Check that it is a RR Type that has an IP Address
                if 'address' in elem.attrib:
                    # Process A, AAAA and PTR Records
                    if re.search(r'PTR|^[A]$|AAAA', elem.attrib['type']):
                        host_names.append(re.search(hostname_pattern, elem.attrib['name']).group(1))

                    # Process NS Records
                    elif re.search(r'NS', elem.attrib['type']):
                        host_names.append(re.search(hostname_pattern, elem.attrib['target']).group(1))

                    # Process SOA Records
                    elif re.search(r'SOA', elem.attrib['type']):
                        host_names.append(re.search(hostname_pattern, elem.attrib['mname']).group(1))

                    # Process MX Records
                    elif re.search(r'MX', elem.attrib['type']):
                        host_names.append(re.search(hostname_pattern, elem.attrib['exchange']).group(1))

                    # Process SRV Records
                    elif re.search(r'SRV', elem.attrib['type']):
                        host_names.append(re.search(hostname_pattern, elem.attrib['target']).group(1))

    elif file_type == 'csv':
        reader = csv.reader(open(file), delimiter=',')
        reader.next()
        for row in reader:
            host_names.append(re.search(hostname_pattern, row[1]).group(1))

    host_names = list(set(host_names))
    # Return list with no empty values
    return filter(None, host_names)


def detect_type(file):
    """
    Function for detecting the file type by checking the first line of the file.
    Returns xml, csv or None.
    """
    ftype = None

    # Get the fist lile of the file for checking
    with open(file) as file:
        firs_line = file.readline()

    # Determine file type based on the fist line content
    if re.search('(xml version)', firs_line):
        ftype = 'xml'
    elif re.search(r'\w*,[^,]*,[^,]*', firs_line):
        ftype = 'csv'
    else:
        raise Exception('Unsupported File Type')
    return ftype


def usage():
    print(f'Version: {__version__}')
    print('DNSRecon output file parser')
    print('Usage: parser.py <options>\n')
    print('Options:')
    print('   -h, --help               Show this help message and exit')
    print('   -f, --file    <file>     DNSRecon XML or CSV output file to parse.')
    print('   -l, --list               Output an unique IP List that can be used with other tools.')
    print('   -i, --ips     <ranges>   IP Ranges in a comma separated list each in formats (first-last)')
    print('                            or in (range/bitmask) for ranges to be included from output.')
    print('                            For A, AAAA, NS, MX, SOA, SRV and PTR Records.')
    print('   -t, --type    <type>     Resource Record Types as a regular expression to filter output.')
    print('                            For A, AAAA, NS, MX, SOA, TXT, SPF, SRV and PTR Records.')
    print('   -s, --str     <regex>    Regular expression between quotes for filtering host names on.')
    print('                            For A, AAAA, NS, MX, SOA, SRV and PTR Records.')
    print('   -n, --name               Return list of unique host names.')
    print('                            For A, AAAA, NS, MX, SOA, SRV and PTR Records.')
    sys.exit(0)


def main():
    #
    # Option Variables
    #
    ip_filter = []
    name_filter = '(.*)'
    type_filter = '(.*)'
    target_list = False
    file = None
    names = False

    #
    # Define options
    #
    try:
        options, args = getopt.getopt(
            sys.argv[1:],
            'hi:t:s:lf:n',
            ['help', 'ips=', 'type=', 'str=', 'list', 'file=', 'name'],
        )

    except getopt.GetoptError as error:
        print_error('Wrong Option Provided!')
        print_error(error)
        return

    #
    # Parse options
    #
    for opt, arg in options:
        if opt in ('-t', '--type'):
            type_filter = arg

        elif opt in ('-i', '--ips'):
            ipranges = arg.split(',')
            for r in ipranges:
                ip_filter.extend(process_range(r))

        elif opt in ('-s', '--str'):
            name_filter = f'({arg})'

        elif opt in ('-l', '--list'):
            target_list = True

        elif opt in ('-f', '--file'):
            # Check if the dictionary file exists
            if os.path.isfile(arg):
                file = arg
            else:
                print_error(f'File {arg} does not exist!')
                exit(1)

        elif opt in ('-r', '--range'):
            ip_list = []
            ip_range = process_range(arg)
            if len(ip_range) > 0:
                ip_list.extend(ip_range)
            else:
                sys.exit(1)
        elif opt in ('-n', '--name'):
            names = True

        elif opt in '-h':
            usage()

    # start execution based on options
    if file:
        if names:
            try:
                found_names = extract_hostnames(file)
                found_names.sort()
                for n in found_names:
                    print_line(n)
            except OSError:
                sys.exit(0)
        else:
            file_type = detect_type(file)
            if file_type == 'xml':
                xml_parse(file, ip_filter, type_filter, name_filter, target_list)
            elif file_type == 'csv':
                csv_parse(file, ip_filter, type_filter, name_filter, target_list)
    else:
        print_error('A DNSRecon XML or CSV output file must be provided to be parsed')
        usage()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#    Unit test for DNSRecon's dnshelper library
#    Author: Filippo Lauria (@filippolauria)
#
#    Copyright (C) 2023 Carlos Perez
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; Applies version 2 of the License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#    See the GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from dnsrecon.lib.dnshelper import DnsHelper
from netaddr import IPAddress
from re import match


class Test_Lib_dnshelper:
    def test_get_a(self):
        helper = DnsHelper("google.com")
        records = helper.get_a("ipv4.google.com")
        for record in records:
            assert record[0] in ["A", "CNAME"]

    def test_get_aaaa(self):
        helper = DnsHelper("google.com")
        records = helper.get_aaaa("ipv6.google.com")
        for record in records:
            assert record[0] in ["AAAA", "CNAME"]

    def test_get_mx(self):
        helper = DnsHelper("google.com")
        records = helper.get_mx()
        for record in records:
            assert record[0] == "MX"

    def test_get_ip(self):
        helper = DnsHelper("google.com")
        records = helper.get_ip("google.com")
        for record in records:
            ip = IPAddress(record[2])
            assert ip.version in [4, 6]  # ~ redundant

    def test_get_txt(self):
        helper = DnsHelper("gmail.com")
        records = helper.get_txt()
        for record in records:
            assert record[0] == "TXT"

    def test_get_ns(self):
        helper = DnsHelper("zonetransfer.me")
        records = helper.get_ns()
        for record in records:
            assert record[0] == "NS"

    def test_get_soa(self):
        helper = DnsHelper("zonetransfer.me")
        records = helper.get_soa()
        for record in records:
            assert record[0] == "SOA"

    def test_get_srv(self):
        helper = DnsHelper("nsztm1.digi.ninja")
        records = helper.get_srv("_sip._tcp.zonetransfer.me")
        for record in records:
            assert record[0] == "SRV"

    def test_zone_transfer(self):
        helper = DnsHelper("zonetransfer.me")
        records = helper.zone_transfer()
        assert len(records) >= 135

    def test_get_ptr(self):
        helper = DnsHelper("zonetransfer.me")
        records = helper.get_ptr("51.79.37.18")
        assert len(records) == 1 and match(r"^.+\.megacorpone\.com$", records[0][1])
